From: Alexandre Chartre <alexandre.chartre@oracle.com>
To: linux-kernel@vger.kernel.org, mingo@kernel.org,
jpoimboe@kernel.org, peterz@infradead.org,
david.laight.linux@gmail.com
Cc: alexandre.chartre@oracle.com
Subject: [PATCH v5 21/30] objtool: Print addresses with alternative instructions
Date: Wed, 19 Nov 2025 15:32:38 +0100 [thread overview]
Message-ID: <20251119143247.3944213-22-alexandre.chartre@oracle.com> (raw)
In-Reply-To: <20251119143247.3944213-1-alexandre.chartre@oracle.com>
All alternatives are disassemble side-by-side when using the --disas
option. However the address of each instruction is not printed because
instructions from different alternatives are not necessarily aligned.
Change this behavior to print the address of each instruction. Spaces
will appear between instructions from the same alternative when
instructions from different alternatives do not have the same alignment.
Signed-off-by: Alexandre Chartre <alexandre.chartre@oracle.com>
---
tools/objtool/disas.c | 35 +++++++++++++++++++++++------------
1 file changed, 23 insertions(+), 12 deletions(-)
diff --git a/tools/objtool/disas.c b/tools/objtool/disas.c
index ae69bef2eb37e..6083a64f6ae49 100644
--- a/tools/objtool/disas.c
+++ b/tools/objtool/disas.c
@@ -47,7 +47,11 @@ struct disas_alt {
struct alternative *alt; /* alternative or NULL if default code */
char *name; /* name for this alternative */
int width; /* formatting width */
- char *insn[DISAS_ALT_INSN_MAX]; /* alternative instructions */
+ struct {
+ char *str; /* instruction string */
+ int offset; /* instruction offset */
+ } insn[DISAS_ALT_INSN_MAX]; /* alternative instructions */
+ int insn_idx; /* index of the next instruction to print */
};
#define DALT_DEFAULT(dalt) (!(dalt)->alt)
@@ -361,10 +365,9 @@ char *disas_result(struct disas_context *dctx)
disas_print_insn(stdout, dctx, insn, depth, "\n")
/*
* Print a message in the instruction flow. If insn is not NULL then
* the instruction address is printed in addition of the message,
* otherwise only the message is printed. In all cases, the instruction
* itself is not printed.
*/
static int disas_vprint(FILE *stream, struct section *sec, unsigned long offset,
int depth, const char *format, va_list ap)
@@ -607,6 +610,7 @@ static int disas_alt_init(struct disas_alt *dalt,
{
dalt->orig_insn = orig_insn;
dalt->alt = alt;
+ dalt->insn_idx = 0;
dalt->name = alt ? disas_alt_name(alt) : strdup("DEFAULT");
if (!dalt->name)
return -1;
@@ -615,7 +619,8 @@ static int disas_alt_init(struct disas_alt *dalt,
return 0;
}
-static int disas_alt_add_insn(struct disas_alt *dalt, int index, char *insn_str)
+static int disas_alt_add_insn(struct disas_alt *dalt, int index, char *insn_str,
+ int offset)
{
int len;
@@ -626,7 +631,8 @@ static int disas_alt_add_insn(struct disas_alt *dalt, int index, char *insn_str)
}
len = strlen(insn_str);
- dalt->insn[index] = insn_str;
+ dalt->insn[index].str = insn_str;
+ dalt->insn[index].offset = offset;
if (len > dalt->width)
dalt->width = len;
@@ -641,12 +647,14 @@ static int disas_alt_group(struct disas_context *dctx, struct disas_alt *dalt)
{
struct objtool_file *file;
struct instruction *insn;
+ int offset;
char *str;
int count;
int err;
file = dctx->file;
count = 0;
+ offset = 0;
alt_for_each_insn(file, DALT_GROUP(dalt), insn) {
@@ -655,9 +663,10 @@ static int disas_alt_group(struct disas_context *dctx, struct disas_alt *dalt)
if (!str)
return -1;
- err = disas_alt_add_insn(dalt, count, str);
+ err = disas_alt_add_insn(dalt, count, str, offset);
if (err)
break;
+ offset += insn->len;
count++;
}
@@ -685,7 +694,7 @@ static int disas_alt_default(struct disas_context *dctx, struct disas_alt *dalt)
str = strdup(disas_result(dctx));
if (!str)
return -1;
- err = disas_alt_add_insn(dalt, 0, str);
+ err = disas_alt_add_insn(dalt, 0, str, 0);
if (err)
return -1;
@@ -710,9 +719,11 @@ static void disas_alt_print_compact(char *alt_name, struct disas_alt *dalts,
for (i = 0; i < alt_count; i++) {
printf("%*s= %s\n", len, "", dalts[i].name);
for (j = 0; j < insn_count; j++) {
- if (!dalts[i].insn[j])
+ if (!dalts[i].insn[j].str)
break;
- printf("%*s| %s\n", len, "", dalts[i].insn[j]);
+ disas_print(stdout, orig_insn->sec,
+ orig_insn->offset + dalts[i].insn[j].offset, 0,
+ "| %s\n", dalts[i].insn[j].str);
}
printf("%*s|\n", len, "");
}
@@ -811,7 +822,7 @@ static void *disas_alt(struct disas_context *dctx,
for (i = 0; i < alt_count; i++) {
free(dalts[i].name);
for (j = 0; j < insn_count; j++)
- free(dalts[i].insn[j]);
+ free(dalts[i].insn[j].str);
}
free(alt_name);
@@ -361,10 +365,9 @@ char *disas_result(struct disas_context *dctx)
disas_print_insn(stdout, dctx, insn, depth, "\n")
/*
- * Print a message in the instruction flow. If insn is not NULL then
- * the instruction address is printed in addition of the message,
- * otherwise only the message is printed. In all cases, the instruction
- * itself is not printed.
+ * Print a message in the instruction flow. If sec is not NULL then the
+ * address at the section offset is printed in addition of the message,
+ * otherwise only the message is printed.
*/
static int disas_vprint(FILE *stream, struct section *sec, unsigned long offset,
--
2.43.5
next prev parent reply other threads:[~2025-11-19 14:34 UTC|newest]
Thread overview: 35+ messages / expand[flat|nested] mbox.gz Atom feed top
2025-11-19 14:32 [PATCH v5 00/30] objtool: Function validation tracing Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 01/30] objtool: Move disassembly functions to a separated file Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 02/30] objtool: Create disassembly context Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 03/30] objtool: Disassemble code with libopcodes instead of running objdump Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 04/30] tool build: Remove annoying newline in build output Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 05/30] objtool: Print symbol during disassembly Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 06/30] objtool: Store instruction disassembly result Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 07/30] objtool: Disassemble instruction on warning or backtrace Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 08/30] objtool: Extract code to validate instruction from the validate branch loop Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 09/30] objtool: Record symbol name max length Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 10/30] objtool: Add option to trace function validation Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 11/30] objtool: Trace instruction state changes during " Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 12/30] objtool: Improve register reporting " Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 13/30] objtool: Identify the different types of alternatives Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 14/30] objtool: Add functions to better name alternatives Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 15/30] objtool: Improve tracing of alternative instructions Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 16/30] objtool: Do not validate IBT for .return_sites and .call_sites Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 17/30] objtool: Add the --disas=<function-pattern> action Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 18/30] objtool: Preserve alternatives order Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 19/30] objtool: Print headers for alternatives Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 20/30] objtool: Disassemble group alternatives Alexandre Chartre
2025-11-19 14:32 ` Alexandre Chartre [this message]
2025-11-19 14:32 ` [PATCH v5 22/30] objtool: Disassemble exception table alternatives Alexandre Chartre
2025-11-20 17:28 ` Josh Poimboeuf
2025-11-19 14:32 ` [PATCH v5 23/30] objtool: Disassemble jump " Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 24/30] objtool: Fix address references in alternatives Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 25/30] objtool: Provide access to feature and flags of group alternatives Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 26/30] objtool: Function to get the name of a CPU feature Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 27/30] objtool: Improve naming of group alternatives Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 28/30] objtool: Compact output for alternatives with one instruction Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 29/30] objtool: Add wide output for disassembly Alexandre Chartre
2025-11-19 14:32 ` [PATCH v5 30/30] objtool: Trim trailing NOPs in alternative Alexandre Chartre
2025-11-20 17:30 ` [PATCH v5 00/30] objtool: Function validation tracing Josh Poimboeuf
2025-11-20 17:37 ` Alexandre Chartre
2025-11-21 10:28 ` Peter Zijlstra
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20251119143247.3944213-22-alexandre.chartre@oracle.com \
--to=alexandre.chartre@oracle.com \
--cc=david.laight.linux@gmail.com \
--cc=jpoimboe@kernel.org \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@kernel.org \
--cc=peterz@infradead.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®